import sys

help_message = '''
python3 -m pip install cameo_claw --upgrade
python3 -c "
from cameo_claw import it_filter
lst_url=[
'https://cameo-claw-data.vercel.app/device_21152332479_daily_2022-03-30.csv',
'https://cameo-claw-data.vercel.app/device_21152332479_daily_2022-03-31.csv']
int_total=len(lst_url)
target_directory = './data/topic_filter/'
lst_distinct_column = ['deviceId', 'localTime', 'sensorId']
lst_column_match=[
    ['sensorId','pm2_5'],
    ['sensorId','voc']
]
sort_column='localTime'
for int_progress, done_url in it_filter(lst_url, target_directory, lst_distinct_column, lst_column_match, sort_column):
    print(f'int_progress/int_total = {int_progress}/{int_total}')
    print(f'done url: {done_url}')
"
'''


def show_help():
    if len(sys.argv) == 1:
        print(help_message)


if __name__ == '__main__':
    show_help()



import requests

from cameo_claw.file import mkdir


def requests_get(f, url, target_directory):
    mkdir(target_directory)
    r = requests.get(url, stream=True)
    if r.status_code == 200:
        bytes1 = r.raw.read()
        if len(bytes1) > 180:  # size larger than 180 bytes we assume the file is not empty
            return f(bytes1)



import gzip
import os
from io import BytesIO
import polars as pl
from cameo_claw.net import requests_get

import warnings
import pandas as pd

warnings.filterwarnings("ignore")


def bytesio_to_gzip(df, url, target_directory):
    bytesio, filename = bytesio_filename(df, url)
    path = f'{target_directory}{filename}.csv.gz'
    write_gzip(bytesio, path)
    return url


def requests_get_write(target_directory, url, f_write):
    try:
        return requests_get(f_write, url, target_directory)
    except Exception as e:
        print(f'cameo_claw.py,requests_get_write,Exception:{e, url}')


def bytesio_filename(df, url):
    bytesio = BytesIO()
    df.to_csv(bytesio)
    filename = os.path.basename(url)
    filename = filename[:filename.find('.')]
    return bytesio, filename


def write_gzip(bytesio, path):
    with gzip.open(path, 'wb') as f:
        f.write(bytesio.getvalue())


def read_csv_distinct(bytes1, lst_distinct_column):
    df = pl.read_csv(bytes1, infer_schema_length=20000)
    df = df.distinct(subset=lst_distinct_column)
    return df


def read_csv_str(bytes1):
    return pl.read_csv(bytes1, infer_schema_length=0).with_columns(pl.all().cast(pl.Int32, strict=False))



import gzip
import os
from io import BytesIO

from cameo_claw.cameo_claw_reuse import bytesio_to_gzip, requests_get_write, \
    bytesio_filename, write_gzip, read_csv_distinct
from cameo_claw.functional import it_mp_f
import polars as pl
import pandas as pd


def a_download(url, target_directory):
    def write(bytes1):
        filename = os.path.basename(url)
        path = f'{target_directory}{filename}'
        with open(path, 'wb') as f:
            f.write(bytes1)
        return url

    return requests_get_write(target_directory, url, write)


def it_download(lst_url, target_directory):
    return it_mp_f(a_download, [tuple([url, target_directory]) for url in lst_url])


def a_distinct(url, target_directory, lst_distinct_column):
    def write(bytes1):
        df = read_csv_distinct(bytes1, lst_distinct_column)
        return bytesio_to_gzip(df, url, target_directory)

    return requests_get_write(target_directory, url, write)


def it_distinct(lst_url, target_directory, lst_distinct_column):
    return it_mp_f(a_distinct, [tuple([url, target_directory, lst_distinct_column]) for url in lst_url])


def a_group(url, target_directory, lst_distinct_column, lst_group_by_column):
    def write(bytes1):
        df = read_csv_distinct(bytes1, lst_distinct_column)
        g = df.groupby(lst_group_by_column)
        for df in g:
            filename_tail = '_group_' + '-'.join(list(
                map(lambda column:
                    str(df.row(0)[df.find_idx_by_name(column)]).replace('_', '-'),
                    lst_group_by_column)))
            bytesio, filename = bytesio_filename(df, url)
            path = f'{target_directory}{filename}{filename_tail}.csv.gz'
            write_gzip(bytesio, path)
        return url

    return requests_get_write(target_directory, url, write)


def it_group(lst_url, target_directory, lst_distinct_column, lst_group_by_column):
    return it_mp_f(a_group,
                   [tuple([url, target_directory, lst_distinct_column, lst_group_by_column]) for url in lst_url])


def a_filter(url, target_directory, lst_distinct_column, lst_column_match=[], sort_column=''):
    def write(bytes1):
        df = read_csv_distinct(bytes1, lst_distinct_column)
        condition = ''
        for lst_col_val in lst_column_match:
            condition += f"""(pl.col('{lst_col_val[0]}')=='{lst_col_val[1]}') | """
        condition = condition[:-3]
        df = df.filter(eval(condition))
        if sort_column:
            df = df.sort(sort_column)
        bytesio_to_gzip(df, url, target_directory)
        return url

    return requests_get_write(target_directory, url, write)


def it_filter(lst_url, target_directory, lst_distinct_column, lst_column_match, sort_column):
    return it_mp_f(a_filter,
                   [tuple([url, target_directory, lst_distinct_column,
                           lst_column_match, sort_column]) for url in lst_url])


def a_chunk(url, target_directory, lst_distinct_column, lst_column_match=[], sort_column=''):
    def write(bytes1):
        df = pd.read_csv(BytesIO(bytes1), compression='gzip')
        df = df.drop_duplicates(subset=lst_distinct_column)
        # condition = ''
        # for lst_col_val in lst_column_match:
        #     condition += f"""(pl.col('{lst_col_val[0]}')=='{lst_col_val[1]}') | """
        # condition = condition[:-3]
        # df = df.filter(eval(condition))
        if sort_column:
            df = df.sort_values(by=[sort_column])
        bytesio = BytesIO()
        df.to_csv(bytesio, header=False, index=False, compression='gzip')
        return bytesio.getvalue()

    return requests_get_write(target_directory, url, write)


def it_chunk(lst_url, target_directory, lst_distinct_column, lst_column_match, sort_column):
    return it_mp_f(a_chunk,
                   [tuple([url, target_directory, lst_distinct_column,
                           lst_column_match, sort_column]) for url in lst_url])



from multiprocessing import Pool
from tqdm import tqdm
import cameo_claw.istarmap

cameo_claw.istarmap.ignore_no_use_warning()


def it_mp_f(f, tup_param):
    int_progress = 0
    with Pool(25) as p:
        for result in tqdm(p.istarmap(f, tup_param), total=len(tup_param)):
            int_progress += 1
            yield int_progress, result



import gzip
from hashlib import md5
from io import BytesIO

import requests
from fastapi import FastAPI
import uvicorn
from fastapi.middleware.cors import CORSMiddleware
from fastapi.responses import StreamingResponse
from cameo_claw.cameo_claw import it_chunk

from pydantic import BaseModel
import polars as pl

app = FastAPI()
app.add_middleware(CORSMiddleware,
                   allow_origins=['*'],
                   allow_credentials=True,
                   allow_methods=['*'],
                   allow_headers=['*'])


class ItemUrlFilter(BaseModel):
    lst_url: list = [
        'https://cameo-claw-data.vercel.app/device_21152332479_daily_2022-03-30.csv',
        'https://cameo-claw-data.vercel.app/device_21152332479_daily_2022-03-31.csv']
    target_directory: str = './data/topic_filter/'
    lst_distinct_column: list = ['deviceId', 'localTime', 'sensorId']
    lst_column_match: list = [
        ['sensorId', 'pm2_5'],
        ['sensorId', 'voc']
    ]
    sort_column: str = 'localTime'


def it_streaming(i: ItemUrlFilter):
    yield write_csv_header_to_gz(i)
    for int_progress, bytes1 in it_chunk(
            i.lst_url, i.target_directory, i.lst_distinct_column, i.lst_column_match, i.sort_column):
        if bytes1:
            yield bytes1
        else:
            continue


def write_csv_header_to_gz(i):
    r = requests.get(i.lst_url[0], verify=False)
    df = pl.read_csv(r.content)
    bytesio_gz = BytesIO()
    with gzip.open(bytesio_gz, 'wb') as f:
        f.write((','.join(df.columns) + '\n').encode('utf-8'))
    return bytesio_gz.getvalue()


dic_item_url_filter = {}


@app.post("/api/streaming_post/")
def streaming_post(i: ItemUrlFilter):
    md5_hex = md5(str(i).encode()).hexdigest()
    dic_item_url_filter[md5_hex] = i
    return md5_hex


@app.get("/api/streaming_get/")
def streaming_get(md5_hex):
    dic = {'content-disposition': f"attachment; filename*=utf-8''all.csv.gz"}
    return StreamingResponse(it_streaming(dic_item_url_filter[md5_hex]), headers=dic)


def server(host='0.0.0.0', int_port=20419):
    uvicorn.run("cameo_claw.streaming:app", host=host, port=int_port, reload=False, debug=False, workers=1)



from datetime import datetime
from cameo_claw.file import mkdir


def log(s):
    mkdir('./data/log')
    with open('./data/log/log.txt', 'a') as f:
        f.write(f'{datetime.now().strftime("%Y-%m-%d %H:%M:%S")}: {s}\n')
